import argparse
import shutil
from pathlib import Path
from typing import List
import threading

import text_detector


def list_files(directory_path: str):
    try:
        files = []
        for file in Path(directory_path).iterdir():
            if file.is_file():
                files.append(str(file))
        return files
    except FileNotFoundError:
        print(f"Error: The directory '{directory_path}' does not exist.")
        return []
    except PermissionError:
        print(f"Error: Permission denied for accessing '{directory_path}'.")
        return []


def list_directories_recursive(directory_path: str):
    try:
        directories = []
        for entry in Path(directory_path).iterdir():
            if entry.is_dir():
                directories.append(str(entry))
                directories.extend(list_directories_recursive(entry))

        return directories
    except FileNotFoundError:
        print(f"Error: The directory '{directory_path}' does not exist.")
        return []
    except PermissionError:
        print(f"Error: Permission denied for accessing '{directory_path}'.")
        return []


def list_files_recursive(directory_path: str):
    try:
        directories = list_directories_recursive(directory_path)
        directories.append(directory_path)
        files = []
        for directory in directories:
            files.extend(list_files(directory))
        return files
    except FileNotFoundError:
        print(f"Error: The directory '{directory_path}' does not exist.")
        return []
    except PermissionError:
        print(f"Error: Permission denied for accessing '{directory_path}'.")
        return []


def copy_file(source_file, destination_dir):
    source = Path(source_file)
    destination = Path(destination_dir)
    destination.mkdir(parents=True, exist_ok=True)  # Ensure the directory exists
    copied_file = destination / source.name
    shutil.copy(source, copied_file)
    return str(copied_file)


def move_files_to_directory(file_paths: List[str], target_dir_path: str):
    try:
        files = [Path(path) for path in file_paths]
        for file in files:
            destination = target_dir_path + file.name
            file.rename(destination)
            print(destination)

    except PermissionError:
        print(f"Error: Permission denied for accessing '{target_dir_path}'.")


def copy_files_to_directory(file_paths: List[str], target_dir_path: str):
    try:
        files = [Path(path) for path in file_paths]
        for file in file_paths:
            dest = copy_file(file, target_dir_path)

            print(f"Copied {file} to {dest}")

    except PermissionError:
        print(f"Error: Permission denied for accessing '{target_dir_path}'.")


def detect_pattern(path: str, pattern: str, detected_files: list[str]):
    if text_detector.check_simple_regex(path, pattern):
        print(f"pattern {pattern} detected in {path}.")
        # python list.append method is thread safe
        detected_files.append(path)
    else:
        print(f"pattern {pattern} not detected in {path}.")


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("-s", "--source", type=str, required=True, help="Path to the source directory.")
    parser.add_argument("-t", "--target", type=str, required=True, help="Path to the destination directory.")
    parser.add_argument("-r", "--recursive", action="store_true", required=False, default=False,
                        help="Search recursively")
    parser.add_argument("-c", "--copy", action="store_true", required=False, default=False,
                        help="Copy the files instead of removing.")
    parser.add_argument("-p", "--pattern", required=True,
                        help="Simple regex pattern to search for. Only supports literals, * and .")
    args = parser.parse_args()
    detected_files = []
    source_files = []
    threads = []
    if args.recursive:
        source_files = list_files_recursive(args.source)
    else:
        source_files = list_files(args.source)

    for file in source_files:
        thread = threading.Thread(target=detect_pattern, args=(file, args.pattern, detected_files))
        threads.append(thread)
        thread.start()
    for thread in threads:
        thread.join()
    if args.copy:
        copy_files_to_directory(detected_files, args.target)
    else:
        move_files_to_directory(detected_files, args.target)
    print(detected_files)


if __name__ == '__main__':
    main()
